/**
  *  \file planetscentral/dbclient/dbsort.cc
  *  \brief Database Client: Sorting
  */

#include <memory>
#include "planetscentral/dbclient/dbsort.h"
#include "planetscentral/dbclient/dblist.h"
#include "planetscentral/dbclient/dbhash.h"

/** Constructor.
    \param client Client we're operating on
    \param keyName Name of key we're sorting (list/set) */
DbSort::DbSort(DbClient& client, const string_t& keyName)
    : client(client),
      req(new DbRequest())
{
    req->withString("SORT").withString(keyName);
}

DbSort::~DbSort()
{ }

/** Define the pattern to sort by. Call at most once.
    \param pattern sort pattern, a key name containing "*". */
DbSort&
DbSort::by(string_t pattern)
{
    req->withString("BY").withString(pattern);
    return *this;
}

/** Define the pattern to sort by. Call at most once.
    \param pattern key containing "*". */
DbSort&
DbSort::by(const DbKey& pattern)
{
    return by(pattern.getName());
}

/** Define the pattern to sort by. Call at most once.
    \param pattern hash field containing "*". */
DbSort&
DbSort::by(const DbHashField& pattern)
{
    return by(pattern.h.getName() + "->" + pattern.name);
}

/** Get sorted value. Call at most once. Must be used when
    get(string_t) is used to get the actual values to be sorted.
    When no get() function is used, behaves as if a single
    parameter-less get() were part of the operation. */
DbSort&
DbSort::get()
{
    req->withString("GET").withString("#");
    return *this;
}

/** Define external keys to get.
    \param pattern sort pattern, a key name containing "*". */
DbSort&
DbSort::get(string_t pattern)
{
    req->withString("GET").withString(pattern);
    return *this;
}

/** Define external keys to get.
    \param pattern key containing "*". */
DbSort&
DbSort::get(const DbKey& pattern)
{
    return get(pattern.getName());
}

/** Define external keys to get.
    \param pattern hash field containing "*". */
DbSort&
DbSort::get(const DbHashField& pattern)
{
    return get(pattern.h.getName() + "->" + pattern.name);
}

/** Limit output. Call at most once.
    \param start First element to get (0-based)
    \param count Number of elements to get */
DbSort&
DbSort::limit(int32_t start, int32_t count)
{
    req->withString("LIMIT").withInt(start).withInt(count);
    return *this;
}

/** Enable lexicographical sort. Default is numerical. Call at most once. */
DbSort&
DbSort::sortLexicographical()
{
    req->withString("ALPHA");
    return *this;
}

/** Sort reversed (descending). Default is ascending. Call at most once. */
DbSort&
DbSort::sortReversed()
{
    req->withString("DESC");
    return *this;
}

/** Do not sort, just retrieve the keys. Call at most once. */
DbSort&
DbSort::dontSort()
{
    req->withString("BY").withString("nosort");
    return *this;
}

/** Get the result into a StringList_t. Call last after setting up the operation.
    \param out [out] The list to fetch this into. */
void
DbSort::getResult(StringList_t& out)
{
    std::auto_ptr<IntValue> result(client.call(*req));
    toStringListResult(result.get(), out);
}

/** Get the result into an IntList_t. Call last after setting up the operation.
    \param out [out] The list to fetch this into. */
void
DbSort::getResult(IntList_t& out)
{
    std::auto_ptr<IntValue> result(client.call(*req));
    toIntListResult(result.get(), out);
}

/** Get the result as IntValue. Call last after setting up the operation. */
IntValue*
DbSort::getResult()
{
    return client.call(*req);
}

/** Store the result into a list object. Call last after setting up the operation.
    \param out List to store in */
void
DbSort::storeResult(DbList out)
{
    req->withString("STORE").withString(out.getName());
    client.callVoid(*req);
}
