/**
  *  \file planetscentral/pathresolver.cc
  */

#include "planetscentral/pathresolver.h"

static bool
isValidFileName(const string_t& n)
{
    return !n.empty() && n[0] != '.';
}

PathResolver::PathResolver(UserDirectory* base, string_t user)
    : base(base), user(user),
      error(0)
{ }

/** Resolve a path.
    \param path [in] Path to resolve [out] On success, final, unverified component of path.
                On error, failing component.
    \retval true success
    \retval false error. error code is set to 400 (syntactic error), 403 (permission error),
    404 (not found, failing components in path) */
bool
PathResolver::resolvePath(string_t& path)
{
    string_t::size_type i;
    while ((i = path.find('/')) != string_t::npos) {
        /* Path must not start with slash, which is syntactically invalid
           (this also applies to double-slashes) */
        if (i == 0) {
            error = 400;
            return false;
        }

        /* Check item */
        base->read();
        UserDirectory* dir = dynamic_cast<UserDirectory*>(base->find(string_t(path, 0, i)));
        if (!dir) {
            /* It does not exist, or it is not a directory. If the user
               is allowed to list this directory, say him; otherwise,
               just say permission denied, to avoid them guessing
               directory names */
            if (base->getPermissions(user) & UserDirectory::AllowList) {
                error = 404;
            } else {
                error = 403;
            }
            return false;
        }

        /* Success */
        base = dir;
        path.erase(0, i+1);
    }

    /* Check final component for syntactic validity */
    if (!isValidFileName(path)) {
        error = 400;
        return false;
    }

    return true;
}

UserItem*
PathResolver::resolveLeaf(string_t path)
{
    /* This is a stripped-down version of resolvePath */
    if (!isValidFileName(path)) {
        error = 400;
        return 0;
    }

    base->read();
    UserItem* it = base->find(path);
    if (!it) {
        if (base->getPermissions(user) & UserDirectory::AllowList) {
            error = 404;
        } else {
            error = 403;
        }
    }
    return it;
}

bool
PathResolver::checkPrivilege(int priv) const
{
    base->read();
    return (base->getPermissions(user) & priv) == priv;
}

int
PathResolver::getError() const
{
    return error;
}

UserDirectory*
PathResolver::getDirectory() const
{
    return base;
}

UserItem*
PathResolver::resolve(string_t path, int priv)
{
    if (!resolvePath(path)) {
        return 0;
    }
    if (!checkPrivilege(priv)) {
        error = 403;
        return 0;
    }
    return resolveLeaf(path);
}

UserDirectory*
PathResolver::resolveToDirectory(string_t path, int priv)
{
    if (!resolvePath(path)) {
        return 0;
    }
    UserItem* it = resolveLeaf(path);
    if (!it) {
        return 0;
    }
    UserDirectory* dir = dynamic_cast<UserDirectory*>(it);
    if (!dir) {
        error = 404;
        return 0;
    }
    dir->read();
    if ((dir->getPermissions(user) & priv) != priv) {
        error = 403;
        return 0;
    }
    return dir;
}
