/**
  *  \file planetscentral/talk/cmdfolder.cc
  *  \brief Talk: FOLDER commands
  */

#include <algorithm>
#include "planetscentral/talk/cmdfolder.h"
#include "planetscentral/dbclient/reconnect.h"
#include "planetscentral/talk/talkconnection.h"
#include "planetscentral/talk/user.h"
#include "planetscentral/talk/userfolder.h"
#include "planetscentral/talk/talk.h"
#include "planetscentral/talk/userpm.h"
#include "planetscentral/talk/cmdforum.h"

/** Configure a folder. */
static void
configureFolder(UserFolder& folder, IntArgBlock& args)
{
    while (args.getNumArgs() > 0) {
        args.checkArgumentCountAtLeast(2);
        string_t key = toStringResult(args.getNext());
        string_t val = toStringResult(args.getNext());
        folder.header().setField(key, val);
    }
}

/** FOLDERLS command.
    No parameters. */
void
doFolderList(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();
    args.checkArgumentCount(0);

    // Do it
    ReconnectingGuard g(database_rc);
    IntList_t result;
    UserFolder::defaultFolders(database_connection).getAll(result);
    User(database_connection, conn.getUser()).pmFolders().getAll(result);

    // Convert to array and send it
    IntDataSegment ds;
    for (IntList_t::size_type i = 0; i < result.size(); ++i) {
        ds.pushNew(makeIntValue(result[i]));
    }
    conn.sendArray(ds, result.size());
}

/** FOLDERSTAT command.
    Parameters:
    - folder Id */
void
doFolderStat(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();
    args.checkArgumentCount(1);
    int32_t ufid = toIntResult(args.getNext());

    // Do it
    ReconnectingGuard g(database_rc);
    User u(database_connection, conn.getUser());
    UserFolder folder(u, ufid);
    std::auto_ptr<IntValue> result(folder.describe(folder.checkExistance()));
    conn.sendObject(result.get());
}

/** FOLDERMSTAT command.
    Parameters:
    - multiple folder Ids */
void
doFolderMStat(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();

    // Do it
    ReconnectingGuard g(database_rc);
    User u(database_connection, conn.getUser());
    IntDataSegment result;
    while (args.getNumArgs() > 0) {
        int32_t ufid = toIntResult(args.getNext());
        std::auto_ptr<IntValue> tmp;
        try {
            UserFolder folder(u, ufid);
            tmp.reset(folder.describe(folder.checkExistance()));
        }
        catch (...) { }
        result.pushNew(tmp.release());
    }
    conn.sendArray(result, result.size());
}

/** FOLDERNEW command.
    Parameter:
    - name of folder */
void
doFolderNew(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();
    args.checkArgumentCountAtLeast(1);
    string_t name = toStringResult(args.getNext());

    // Do it
    ReconnectingGuard g(database_rc);
    User u(database_connection, conn.getUser());

    // Create the folder
    int32_t newUfid = UserFolder::allocateFolder(u);
    UserFolder folder(u, newUfid);
    u.pmFolders().add(newUfid);

    // Configure the folder
    folder.header().setField("name", name);
    configureFolder(folder, args);

    IntIntIntValue iv(newUfid);
    conn.sendObject(&iv);
}

/** FOLDERRM command.
    Perameter:
    - folder Id */
void
doFolderRemove(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();
    args.checkArgumentCount(1);
    int32_t ufid = toIntResult(args.getNext());

    // Do it
    ReconnectingGuard g(database_rc);
    User u(database_connection, conn.getUser());
    UserFolder folder(u, ufid);

    // Try to remove it.
    if (!u.pmFolders().remove(ufid)) {
        // Remove failed, this is not a user folder. Either it did not exist,
        // or it is a global default folder.
        conn.sendSuccess("0");
    } else {
        // Unlink all messages
        IntList_t msgs;
        folder.messages().getAll(msgs);
        for (IntList_t::iterator i = msgs.begin(); i != msgs.end(); ++i) {
            UserPM(database_connection, *i).removeReference();
        }

        // Remove the folder
        folder.remove();
        conn.sendSuccess("1");
    }
}

/** FOLDERSET command.
    Parameter:
    - folder Id
    - pairs of key/value */
void
doFolderSet(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();
    args.checkArgumentCountAtLeast(1);
    int32_t ufid = toIntResult(args.getNext());

    // Do it
    ReconnectingGuard g(database_rc);
    User u(database_connection, conn.getUser());
    UserFolder folder(u, ufid);
    folder.checkExistance();
    configureFolder(folder, args);

    conn.sendSuccess("OK");
}

/** FOLDERLSPM command.
    Parameter:
    - folder Id
    - list parameters */
void
doFolderListPMs(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();
    ListParams parms;
    parms.parse(args, true, ListParams::pmSortKeys);

    // Do it
    ReconnectingGuard g(database_rc);
    User u(database_connection, conn.getUser());
    UserFolder folder(u, parms.getId());
    folder.checkExistance();
    parms.exec(conn, folder.messages());
}
