/**
  *  \file planetscentral/talk/userfolder.cc
  *  \brief Talk: User Folders (for PMs)
  */

#include <memory>
#include <stdexcept>
#include "planetscentral/talk/userfolder.h"
#include "planetscentral/talk/user.h"
#include "planetscentral/talk/db.h"
#include "int/array.h"

/** Constructor.
    \param user User to create this folder for
    \param userFolderId User's folder Id */
UserFolder::UserFolder(User& user, int32_t userFolderId)
    : userFolderSet(user.pmFolders()),
      userFolder(user.pmFolderData().subtree(userFolderId)),
      userFolderId(userFolderId)
{ }

/** Check existance of this folder.
    \retval true this is a user folder
    \retval false this is a default folder */
bool
UserFolder::checkExistance()
{
    if (userFolderSet.contains(userFolderId)) {
        return true;
    } else if (defaultFolders(userFolder.getClient()).contains(userFolderId)) {
        return false;
    } else {
        throw std::runtime_error("404 Folder does not exist");
    }
}

/** Set of messages in this folder. */
DbIntSet
UserFolder::messages()
{
    return userFolder.intSetKey("messages");
}

/** Header of this folder. This returns the user-specific header,
    even if the folder does not have one. A write access would create it.
    See getHeader() for details. */
DbHash
UserFolder::header()
{
    return userFolder.hashKey("header");
}

/** Flag for unread messages in this folder.
    This is vaguely specified to allow an unread message count later, but
    so far it is implemented as a flag only. */
DbHashIntField
UserFolder::unreadMessages()
{
    return header().intField("unread");
}

/** Get header element. Folders can have user-specific and global
    default headers. The latter are used for folders every user has.
    This looks into the user-specific header first, then into the
    global one. That is, if the user has renamed his outbox, this will
    return the new name; if he has not touched it, it will return the
    global name. */
string_t
UserFolder::getHeader(string_t key)
{
    std::auto_ptr<IntValue> result(header().getFieldRaw(key));
    if (result.get() == 0) {
        result.reset(defaultHeader().getFieldRaw(key));
    }
    return toStringResult(result.get());
}

/** Describe this folder.
    \param isUser true iff this is a user folder */
IntValue*
UserFolder::describe(bool isUser)
{
    Ptr<IntArrayData> ad(new IntArrayData());

    // Name
    ad->content.pushNew(makeStringValue("name"));
    ad->content.pushNew(makeStringValue(getHeader("name")));
    ad->content.pushNew(makeStringValue("description"));
    ad->content.pushNew(makeStringValue(getHeader("description")));

    // Messages
    ad->content.pushNew(makeStringValue("messages"));
    ad->content.pushNew(makeIntValue(messages().size()));
    ad->content.pushNew(makeStringValue("unread"));
    ad->content.pushNew(makeIntValue(unreadMessages().get()));

    // Status
    ad->content.pushNew(makeStringValue("fixed"));
    ad->content.pushNew(makeIntValue(!isUser));

    ad->addDimension(ad->content.size());
    return new IntArray(ad);
}

/** Remove this folder.
    Assumes that it has already been unlinked, as well as all the messages. */
void
UserFolder::remove()
{
    messages().remove();
    header().remove();
}

/** Allocate a new folder Id. */
int32_t
UserFolder::allocateFolder(User& user)
{
    /* We want the user's first folder to be 100. Writing it this way
       makes sure that there will not be a folder below 100, even with
       concurrent accesses, although if there actually are concurrent
       accesses, initial folders might be a little bigger than expected.
       But this is not a problem. */
    int32_t result = ++user.pmFolderCount();
    if (result < 100) {
        result = user.pmFolderCount() += (100 - result);
    }
    return result;
}

/** Get list of default folders. */
DbIntSet
UserFolder::defaultFolders(DbClient& client)
{
    return DbSubtree(client, DEFAULT_FOLDER_ROOT).intSetKey("all");
}

/** Get global default header. */
DbHash
UserFolder::defaultHeader()
{
    return DbSubtree(userFolder.getClient(), DEFAULT_FOLDER_ROOT).subtree(userFolderId).hashKey("header");
}
